const Me      = imports.misc.extensionUtils.getCurrentExtension();
const UiGroup = imports.ui.main.uiGroup;

const Prefs = Me.imports.prefs;
const Ui    = Me.imports.ui;
const View  = Me.imports.view;

let keys    = {};
let corners = {};
let undos   = [];

function init() {
    function corner(left, page) {
        let [ corner, ripples, animation ] = Ui.animation(left);
        return {
            enable : function() {
                ripples.concat(corner).forEach(function(actor, i, a) {
                    UiGroup.add_actor(actor);
                    undos.push(
                        function() {
                            UiGroup.remove_actor(actor);
                        }
                    );
                });

                undos.push(
                    Prefs.connect(corner, 'enter-event', animation.show),
                    Prefs.connect(corner, 'enter-event', page.toggle)
                );
            },
            workaround : animation.show
        };
    }

    function key(register, page) {
        return {
            enable : function() {
                undos.push(
                    register(page.toggle)
                );
            }
        };
    }

    corners = {
        left  : corner(Ui.LEFT, View.applications),
        right : corner(Ui.RIGHT, View.workspaces)
    };

    keys = {
        apps   : key(Prefs.onViewApps, View.applications),
        search : key(Prefs.onSearch, View.search)
    };
}

function enable() {
    for each (let corner in corners) {
        corner.enable();
    }
    corners.right.workaround();

    for each (let key in keys) {
        key.enable();
    }
}

function disable() {
    while (undos.length > 0) {
        undos.pop()();
    }
}
